package jp.gr.java_conf.abagames.comraid;

import jp.gr.java_conf.abagames.comraid.shapes.*;
import jp.gr.java_conf.abagames.util.*;
import jp.gr.java_conf.abagames.util.gl4j.*;
import gl4java.*;
import java.util.Random;

/**
  Battery operation
  */
public class Battery extends LayerdSprite implements IMove {
  static private BatteryShape shape = new BatteryShape();

  private SceneUI sceneUI;
  private Comraid comraid;
  private float lfSpin, rtSpin, lfDeg, rtDeg;
  private float ntLfDeg, ntRtDeg;
  public int dapCnt;
  private float md;
  public float mx, my;
  public float lsrScl;
  public int stgLyr;
  public float mvLyr;
  private int[] elScr = new int[3];
  private int[] elAddScr = new int[3];

  static final private int REPLAYMAX = 12800;
  public byte[] replay = new byte[REPLAYMAX];
  private int rplIdx;

  public Battery(GLFunc gl, SceneUI sceneUI, Comraid comraid) {
    super(shape, gl);
    this.sceneUI = sceneUI;
    this.comraid = comraid;
    start();
  }

  final public int LYRMVTIME = 25, LYRTIME = 600;

  final private int ELSCR = 10;

  final private int KSSCR = 10, KSDEG = 180;

  private boolean demof = false;

  public void start() {
    //x = 0; y = -9;
    startStage();
    restart();
    demof = false;
  }

  private Random rnd = new Random();

  public void startDemo(byte[] rp) {
    start();
    demof = true;
    mx = 0; my = 0;
    for ( int i=0 ; i<replay.length ; i++ ) {
      replay[i] = rp[i];
    }
  }

  public void resetEnLine() {
    for ( int i=0 ; i<3 ; i++ ) {
      elScr[i] = elAddScr[i] = ELSCR*2;
    }
  }

  private void startStage() {
    x = 0; y = -9;
    SPEED = 0.3f;
    stCnt = 0;
    lsrOst = 0;
    resetEnLine();
    stgLyr = 0;
    mvLyr = 0;
    rplIdx = 0;
    for ( int i=0 ; i<replay.length ; i++ ) {
      replay[i] = 0;
    }
  }

  private void restart() {
    red = 0.2f; green = 0.9f; blue = 0.6f;
    lfSpin = rtSpin = lfDeg = rtDeg = 0;
    ntLfDeg = ntRtDeg = KSDEG;
    md = 0;
    dapCnt = -180;
    dstDeg = 0;
    xScl = yScl = 1.0f;
    lsrScl = 0;
    for ( int i=0 ; i<3 ; i++ ) {
      elScr[i] /= 2; elScr[i] -= elScr[i]%10;
      elAddScr[i] /= 2; elAddScr[i] -= elAddScr[i]%10;
      if ( elScr[i] < 10 ) elScr[i] = 10;
      if ( elAddScr[i] < 10 ) elAddScr[i] = 10;
    }
    pvClr = pvpvClr = -1;
  }

  private float SPEED = 0.3f;
  final private float SLANTSPEED = 0.707f;

  private int stCnt = 0;
  private float lsrOst = 0;
  private int pvClr, pvpvClr;

  public void passEngLine(int color) {
    comraid.playSe(3);
    if ( pvpvClr == EnergyLine.RED
     && pvClr == EnergyLine.BLUE
     && color == EnergyLine.YELLOW ) {
      elScr[EnergyLine.YELLOW] = 10000;
      elAddScr[EnergyLine.YELLOW] = 1000;
    }
    for ( int i=0 ; i<3 ; i++ ) {
      if ( i == color ) {
        comraid.score += elScr[i];
        comraid.addLetters(Integer.toString(elScr[i]), x, y, layer, 0.4f, 45);
        elScr[i] += elAddScr[i]; elAddScr[i] += ELSCR;
        if ( elScr[i] > 100000 ) elScr[i] = 100000;
        if ( elAddScr[i] > 10000 ) elAddScr[i] = 10000;
      } else {
        elScr[i] /= 2 ;elScr[i] -= elScr[i]%10;
        if ( elScr[i] < 10 ) elScr[i] = 10;
      }
    }
    pvpvClr = pvClr; pvClr = color;
  }

  private void miss() {
    comraid.playSe(4);
    dapCnt = 60; md = -24;
  }

  public void hit() {
    if ( dapCnt == -1 ) {
      miss();
    }
  }

  public void move() {
    if ( dapCnt < 0 ) {
      if ( demof ) {
        byte rp = replay[rplIdx];
        if ( rp == 1 ) {
          if ( (rnd.nextInt()&31) == 0 ) {
            mx = ((rnd.nextInt()&1)*2-1)*SPEED;
            my = ((rnd.nextInt()&1)*2-1)*SPEED;
            if ( mx!=0 && my!=0 ) {
              mx *= SLANTSPEED; my*=SLANTSPEED;
            }
          }
        } else {
          mx = 0; my = 0;
          if ( (rp&128)==128 ) {
            my = SPEED;
          }
          if ( (rp&64)==64 ) {
            my = -SPEED;
          }
          if ( (rp&32)==32 ) {
            mx = -SPEED;
          }
          if ( (rp&16)==16 ) {
            mx = SPEED;
          }
          if ( mx!=0 && my!=0 ) {
            mx *= SLANTSPEED; my*=SLANTSPEED;
          }
        }
      } else {
        mx = 0; my = 0;
        if ( sceneUI.upKey ) {
          my =  SPEED;
          replay[rplIdx] |= 128;
        }
        if ( sceneUI.dwKey ) {
          my = -SPEED;
          replay[rplIdx] |= 64;
        }
        if ( sceneUI.lfKey ) {
          mx = -SPEED;
          replay[rplIdx] |= 32;
        }
        if ( sceneUI.rtKey ) {
          mx =  SPEED;
          replay[rplIdx] |= 16;
        }
        if ( mx!=0 && my!=0 ) {
          mx *= SLANTSPEED; my*=SLANTSPEED;
        }
      }
      x += mx; y += my;
      if ( x>Comraid.frmWidth-1 || x<-Comraid.frmWidth+1 ) {
        x -= mx; mx = 0;
      }
      if ( y>Comraid.frmHeight-1 || y<-Comraid.frmHeight+1 ) {
        y -= my; my = 0;
      }
      if ( demof ) {
        byte rp = replay[rplIdx];
        if ( rp == 1 ) {
          if ( stCnt <= 0 ) {
            comraid.addShot(x, y);
            stCnt = 3;
          }
        } else {
          if ( (rp&4)==4 ) {
            comraid.addLaser(lsrOst);
            lsrOst += 0.2f;
            if ( lsrOst >= 1.0f ) {
              lsrOst -= 1.0f;
              comraid.addLaser(lsrOst);
            }
            if ( lsrScl < 1.0f ) lsrScl += 0.1f;
            SPEED = 0.2f;
          } else {
            if ( (rp&8)==8 ) {
              comraid.addShot(x, y);
            }
            if ( lsrScl > 0 ) {
              lsrScl -= 0.2f; if ( lsrScl < 0 ) lsrScl = 0;
            }
            SPEED = 0.3f;
          }
        }
      } else {
        if ( sceneUI.lsKey ) {
          replay[rplIdx] |= 4;
          comraid.addLaser(lsrOst);
          lsrOst += 0.2f;
          if ( lsrOst >= 1.0f ) {
            lsrOst -= 1.0f;
            comraid.addLaser(lsrOst);
          }
          if ( lsrScl < 1.0f ) lsrScl += 0.1f;
          SPEED = 0.2f;
        } else {
          if ( sceneUI.stKey && stCnt<=0 ) {
            replay[rplIdx] |= 8;
            comraid.addShot(x, y);
            stCnt = 3;
          }
          if ( lsrScl > 0 ) {
            lsrScl -= 0.2f; if ( lsrScl < 0 ) lsrScl = 0;
          }
          SPEED = 0.3f;
        }
      }
      if ( stCnt > 0 ) {
        stCnt--;
      }
      float ofx, ofy;
      for ( int i=comraid.BULLETMAX-1 ; i>=0 ; i-- ) {
        Bullet blt = comraid.bullets[i];
        if ( blt.exists() && blt.layer==1 ) {
          ofx = x-blt.x; ofy = y-blt.y;
          if ( Math.abs(ofx) <= 1.3f ) {
            if ( Math.abs(ofy)<=0.2f && Math.abs(ofx)<=0.2f && dapCnt == -1 ) {
              miss();
            } else if ( ofy<=0.3 && ofy>=-1.3f ) {
              if ( ofx>0 ) lfSpin += 10f;
              else rtSpin += 10f;
            }
          }
        }
      }
      lfSpin *= 0.96f; rtSpin *= 0.96f;
      lfDeg += lfSpin; rtDeg += rtSpin;
      while ( lfDeg >= ntLfDeg ) {
        comraid.score += KSSCR;
        ntLfDeg += KSDEG;
      }
      while ( rtDeg >= ntRtDeg ) {
        comraid.score += KSSCR;
        ntRtDeg += KSDEG;
      }
      shape.setLeftDeg(lfDeg);
      shape.setRightDeg(rtDeg);
      if ( dapCnt < -1 ) {
        switch ( dapCnt&3 ) {
          case 0:
            red = 0.1f; green = 0.5f; blue = 0.3f;
            break;
          case 2:
            red = 0.2f; green = 0.9f; blue = 0.6f;
            break;
        }
        dapCnt++;
      }
    } else {
      dstDeg += md;
      if ( dapCnt > 40 ) {
        xScl *= 0.9f; yScl *= 1.3f;
      } else {
        xScl = 0.95f; yScl *= 0.8f;
      }
      dapCnt--;
      red = blue = (60-dapCnt)*0.015f;
      if ( dapCnt <= 0 ) {
        comraid.decLeft();
        restart();
      }
    }

    int lyt = comraid.time;
    if ( lyt == LYRTIME-LYRMVTIME ) {
      mvLyr = 0.95f/LYRMVTIME;
    } else if ( lyt >= LYRTIME ) {
      mvLyr = 0; stgLyr++;
      comraid.time = 0;
    }
    rplIdx++;
  }

  public boolean exists() {
    return true;
  }
}

