package jp.gr.java_conf.abagames.comraid;

import jp.gr.java_conf.abagames.comraid.crafts.*;
import jp.gr.java_conf.abagames.comraid.shapes.*;
import jp.gr.java_conf.abagames.util.*;
import jp.gr.java_conf.abagames.util.gl4j.*;
import jp.gr.java_conf.abagames.util.jmf.*;
import java.util.Random;
import java.net.URL;
import java.awt.EventQueue;
import gl4java.*;

/**
  Comraid boot class
  */
public class Comraid implements ITime, IInit {
  static public Random rnd;
  final static public float frmWidth = 12.0f, frmHeight = 12.0f;

  private boolean shakef = true;
  private boolean bgmf = false;

  private int frmCnt = 0;

  private int duration = 33;
  private SceneUI sceneUI;
  private Scene scene;
  private Timer timer;

  final public int BULLETMAX = 96, CRAFTMAX = 128,
   SHOTMAX = 48, LASERMAX = 32, ELMAX = 32, FRMAX = 11, LTMAX = 32,
   LBMAX = 4;
  final private int OBJMAX
   = (BULLETMAX+CRAFTMAX+SHOTMAX+LASERMAX+ELMAX+FRMAX+LTMAX+LBMAX);
  public Bullet[] bullets = new Bullet[BULLETMAX];
  private Craft[] crafts = new Craft[CRAFTMAX];
  public Shot[] shots = new Shot[SHOTMAX];
  public Laser[] lasers = new Laser[LASERMAX];
  public EnergyLine[] enLines = new EnergyLine[ELMAX];
  private Battery battery;
  private LevelFrame[] frames = new LevelFrame[FRMAX];
  private Letters[] letters = new Letters[LTMAX];
  private LeftBattery[] leftBtrs = new LeftBattery[LBMAX];
  public int score, pvScr;
  private Letters scrLt;
  final private int STAGENMB = 6;
  private int stage;
  private int left, leftMax = 2, credit;
  private int nextEtdScr;
  final private int FIRSTEXTEND = 500000, SECONDEXTEND = 2000000;
  private boolean stgSlcf;
  private int stgstScore;

  private LayerdSprite[] sprites = new LayerdSprite[OBJMAX*3];

  private MdPlayer bgm;
  private SePlayer sePlayer;

  private SaveDataManager sdManager;
  private SaveData saveData;

  private int mode = 1;

  public void start() {
    sdManager = new SaveDataManager();
    sdManager.load();
    saveData = sdManager.getSaveData();

    try {
      sePlayer = new SePlayer();
    } catch ( NoClassDefFoundError e ) {
      saveData.sef = false;
    }

    sceneUI = new SceneUI(this, saveData.sceneUIRect);
    sceneUI.show();
  }

  private void loadBgm(String filename) {
    if ( !saveData.bgmf ) {
      bgmf = false;
      return;
    }
    try {
      bgm = new MdPlayer(filename, true);
      bgmf = true;
    } catch ( Exception e ) {
      e.printStackTrace();
      saveData.bgmf = bgmf = false;
    } catch ( NoClassDefFoundError e ) {
      saveData.bgmf = bgmf = false;
    }
  }

  public void init() {
    scene = sceneUI.getScene();
    if ( saveData.joystickf ) {
      sceneUI.initJoystick();
      sceneUI.enable(saveData.joystickf);
    }

    battery = new Battery(scene.glFunc, sceneUI, this);
    battery.layer = 1.0f;
    for ( int i=0 ; i<bullets.length ; i++ ) {
      bullets[i] = new Bullet(scene.glFunc);
    }
    crafts[0] = new Craft(scene.glFunc, this, battery, true);
    for ( int i=1 ; i<crafts.length ; i++ ) {
      crafts[i] = new Craft(scene.glFunc, this, battery, false);
    }
    for ( int i=0 ; i<shots.length ; i++ ) {
      shots[i] = new Shot(scene.glFunc);
    }
    for ( int i=0 ; i<lasers.length ; i++ ) {
      lasers[i] = new Laser(scene.glFunc, battery);
    }
    for ( int i=0 ; i<enLines.length ; i++ ) {
      enLines[i] = new EnergyLine(scene.glFunc, battery);
    }
    for ( int i=0 ; i<frames.length ; i++ ) {
      frames[i] = new LevelFrame(scene.glFunc, i);
    }
    for ( int i=0 ; i<letters.length ; i++ ) {
      letters[i] = new Letters(scene.glFunc);
    }
    for ( int i=0 ; i<leftBtrs.length ; i++ ) {
      leftBtrs[i] = new LeftBattery(scene.glFunc, i);
    }

    startTitle();

    timer = new Timer(this);
    timer.activate(duration, keyPriorityf);
  }

  public void decLeft() {
    if ( mode == 1 ) return;

    left--;
    if ( left < 0 ) {
      startGameover();
    } else {
      setLeftBattery(left);
    }
  }

  public void setLeftBattery(int n) {
    left = n;
    for ( int i=0 ; i<LBMAX ; i++ ) {
      if ( i < n ) {
        leftBtrs[i].existf = true;
      } else {
        leftBtrs[i].existf = false;
      }
    }
  }

  private void startGameCfg(int stage) {
    this.stage = stage;
    score = pvScr = 0;
    setLeftBattery(leftMax);
    credit = saveData.creditMax;
    nextEtdScr = FIRSTEXTEND;
    startStage();
    sceneUI.psKey = false;
    gmMaxScr = 0;
  }

  private void startGame() {
    stgSlcf = false;
    startGameCfg(1);
  }

  private void startGame(int stage) {
    stgSlcf = true;
    startGameCfg(stage);
  }

  private StageManager sm = new StageManager(this);

  public int time;
  public int bssTime;
  private int stgEndTime;

  private Letters stgLt, stgLt2;
  private String stageMessage;
  private int stgMsgIdx;

  private void clearSprites() {
    for ( int i=0 ; i<bullets.length ; i++ ) {
      bullets[i].clear();
    }
    for ( int i=0 ; i<crafts.length ; i++ ) {
      crafts[i].clear();
    }
    for ( int i=0 ; i<shots.length ; i++ ) {
      shots[i].clear();
    }
    for ( int i=0 ; i<lasers.length ; i++ ) {
      lasers[i].clear();
    }
    for ( int i=0 ; i<enLines.length ; i++ ) {
      enLines[i].clear();
    }
    for ( int i=0 ; i<letters.length ; i++ ) {
      letters[i].clear();
    }
    resetFrames(frames);
  }

  private StageData setStageData() {
    resetViewpoint(false);
    rnd = new Random(stageSeed);
    StageData sd = sm.makeStageData(stage);
    bssTime = -(sd.getBossEngTime());
    stgEndTime = sd.getStageEndTime();
    stageMessage = sd.getMessage();
    clearSprites();
    LayerdSprite.stopBossShake();
    return(sd);
  }

  private Letters pusLt;

  public void startPause() {
    pusLt = addLetters("PAUSE", 0, 8, 1, 1, 1);
    mode = 4;
  }

  public void startEnding() {
    resetViewpoint(true);
    addLetters("CONRGATULATIONS", 0, 5, 1, 1, 99999);
    ttlTime = 10000;
    mode = 2;
  }

  private int gmMaxScr;

  public void startGameover() {
    if ( score > gmMaxScr ) gmMaxScr = score;
    resetViewpoint(true);
    if ( credit > 0 ) {
      cntCnt = addLetters("9", 0, 0, 1, 3, 1);
      stgLt = addLetters("CONTINUE", 0, 8, 1, 1, 1);
      addLetters("CREDIT "+credit, 0, -11, 1, 0.4f, 1);
      ttlTime = -300;
    } else {
      cntCnt = addLetters(" ", 0, 0, 1, 3, 1);
      stgLt = addLetters(" ", 0, 8, 1, 1, 1);
      if ( credit == 0 ) saveData.creditMax++;
      ttlTime = -1;
    }
    mode = 2;
    arwPushed = btnPushed = true;
  }

  private int entRnk;
  private Letters nmLt, nmChrLt, rnkScrLt;
  private int nmIdx;
  private char[] rnkNm = new char[3];
  final private char[] nmChrs = {
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N',
    'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '0', '1',
    '2', '3', '4', '5', '6', '7', '8', '9', '.', ' '};

  public void startEntry() {
    clearSprites();
    if ( stgSlcf ) {
      startTitle();
      return;
    }
    for ( entRnk=0 ; entRnk<10 ; entRnk++ ) {
      if ( gmMaxScr > saveData.rnkScores[entRnk] ) {
        for ( int i=8 ; i>=entRnk ; i-- ) {
          for ( int j=0 ; j<3 ; j++ ) {
            saveData.rnkNames[i+1][j] = saveData.rnkNames[i][j];
          }
          saveData.rnkScores[i+1] = saveData.rnkScores[i];
        }
        break;
      }
    }
    if ( entRnk >= 10 ) {
      startTitle();
      return;
    }
    saveData.rnkScores[entRnk] = gmMaxScr;
    for ( int i=0 ; i<3 ; i++ ) {
      saveData.rnkNames[entRnk][i] = 'A';
    }
    stgLt = addLetters(rnkStrs[entRnk], 0, 6, 1, 1, 99999);
    nmChrLt = addLetters("A", 2.5f, 0, 1, 2, 99999);
    nmLt = addLetters("", 0, 0, 1, 1.5f, 99999);
    rnkScrLt = addLetters(Integer.toString(gmMaxScr), 0, -6, 1, 1, 99999);
    nmIdx = 0;
    rnkNm[0] = rnkNm[1] = rnkNm[2] = 0;
    ttlTime = 0;
    mode = 5;
    arwPushed = btnPushed = true;
    rptCnt = 32;
  }

  final private int MENUIDXMAX = 8;
  private Letters[] menuLetters = new Letters[MENUIDXMAX];
  private int selectedStage = 1;
  private int menuIdx;

  public void startMenu() {
    resetViewpoint(true);
    clearSprites();
    battery.x = 99999;
    menuLetters[0] = addLetters("GAME START", 0, 4, 1, 0.8f, 99999);
    menuLetters[1] = addLetters("STAGE SELECT "+selectedStage, 0, 2, 1, 0.5f, 99999);
    menuLetters[2] = addLetters("LAYER " + (saveData.shwLyrNmb-1), 0, 0, 1, 0.5f, 99999);
    menuLetters[3] = addLetters("FRAME SKIP " + saveData.frameSkip, 0, -2, 1, 0.5f, 99999);
    String joyStr;
    if ( saveData.joystickf ) {
      joyStr = "JOYSTICK ON";
    } else {
      joyStr = "JOYSTICK OFF";
    }
    menuLetters[4] = addLetters(joyStr, 0, -4, 1, 0.5f, 99999);
    String bgmStr;
    if ( saveData.bgmf ) {
      bgmStr = "BGM ON";
    } else {
      bgmStr = "BGM OFF";
    }
    menuLetters[5] = addLetters(bgmStr, 0, -6, 1, 0.5f, 99999);
    String seStr;
    if ( saveData.sef ) {
      seStr = "SE ON";
    } else {
      seStr = "SE OFF";
    }
    menuLetters[6] = addLetters(seStr, 0, -8, 1, 0.5f, 99999);
    menuLetters[7] = addLetters("EXIT", 0, -10, 1, 0.5f, 99999);
    menuIdx = 0;
    mode = 3;
    arwPushed = btnPushed = true;
    System.gc();
  }

  private int ttlTime;
  private int demoStg = 1;

  private int stageSeed = 0;

  public void startTitle() {
    stage = demoStg;
    demoStg++;
    if ( demoStg > saveData.stgNmb ) demoStg = 1;
    StageData sd = setStageData();
    time = -60;
    battery.startDemo(saveData.replays[stage-1]);
    scrLt = addLetters("0", 0, 13, 1, 0.6f, 99999);
    stgLt = addLetters("COMRAID", 0, 5, 1, 2, 600);

    mode = 1;
    ttlTime = 0;
    score = 0; nextEtdScr = Integer.MAX_VALUE;
    System.gc();
  }

  public void startStage() {
    if ( saveData.stgNmb < stage ) {
      saveData.stgNmb = stage;
    }
    StageData sd = setStageData();
    time = -60;
    battery.start();
    scrLt = addLetters(Integer.toString(score), 0, 13, 1, 0.6f, 99999);
    stgLt = addLetters("STREAM "+stage, 0, 0, 1, 1.5f, 180);
    loadBgm(sd.getBgmFileName());
    mode = 0;
    stgstScore = score;
    System.gc();

    if ( bgmf ) bgm.play();
  }

  public void stopBgm() {
    if ( bgmf && bgm != null ) bgm.stop();
  }

  public void clearStage() {
    stopBgm();
    int stgSc = score-stgstScore;
    if ( stgSc > saveData.stgScores[stage-1] ) {
      saveData.stgScores[stage-1] = stgSc;
      for ( int i=0 ; i<saveData.replays[stage-1].length ; i++ ) {
        saveData.replays[stage-1][i] = battery.replay[i];
      }
    }
    time = bssTime = -99999;
    battery.mvLyr = -0.1f;
    battery.dapCnt = -99999;
  }

  public void destroy() {
    saveData.sceneUIRect = sceneUI.getBounds();
    sdManager.save();

    if ( timer != null ) {
      timer.deactivate();
    }
    if ( scene != null ) {
      scene.destroy();
    }
    System.exit(0);
  }

  private int cnt = 0;
  private int sIdx[] = new int[3];

  private void setLayerdSprites(IMove[] objs) {
    int sln = saveData.shwLyrNmb;
    if ( sln > 2 ) {
      for ( int i=objs.length-1 ; i>=0 ; i-- ) {
        if ( objs[i].exists() ) {
          LayerdSprite ls = (LayerdSprite)objs[i];
          if ( ls.layer == 1 ) {
            sprites[sIdx[1]] = ls; sIdx[1]++;
          } else if ( ls.layer<1 && ls.layer>0 ) {
            sprites[sIdx[0]] = ls; sIdx[0]++;
          } else if ( ls.layer < sln ) {
            sprites[sIdx[2]] = ls; sIdx[2]++;
          }
        }
      }
    } else {
      for ( int i=objs.length-1 ; i>=0 ; i-- ) {
        if ( objs[i].exists() ) {
          LayerdSprite ls = (LayerdSprite)objs[i];
          if ( ls.layer == 0 ) {
          } else if ( ls.layer == 1 ) {
            sprites[sIdx[1]] = ls; sIdx[1]++;
          } else if ( ls.layer < 1 ) {
            sprites[sIdx[0]] = ls; sIdx[0]++;
          } else if ( ls.layer < sln ) {
            sprites[sIdx[2]] = ls; sIdx[2]++;
          }
        }
      }
    }
  }

  private void setLayerdSprite(IMove obj) {
    if ( obj.exists() ) {
      LayerdSprite ls = (LayerdSprite)obj;
      if ( ls.layer < 1 ) {
        sprites[sIdx[0]] = ls; sIdx[0]++;
      } else if ( ls.layer == 1 ) {
        sprites[sIdx[1]] = ls; sIdx[1]++;
      } else {
        sprites[sIdx[2]] = ls; sIdx[2]++;
      }
    }
  }

  synchronized private void waitFrame() {
    //if ( EventQueue.isDispatchThread() ) return;
    try {
      wait();
    //} catch ( InterruptedException e ) {}
    } catch ( Exception e ) {}
  }

  synchronized final public void notifyFrame() {
    notifyAll();
  }

  private boolean viewRstf = false;

  private void resetViewpoint(boolean f) {
    viewRstf = f;
  }

  final public void display() {
    if ( time<0 && !viewRstf ) {
      if ( time >= -60 ) {
        int tt = time+1;
        tt *= tt; tt /= 10;
        scene.glFunc.glPushMatrix();
        scene.setViewpoint(-tt*0.3f, tt*0.2f-1.0f, tt+9.0f,  0, 0, -5.0f);
      } else {
        int tt = 99999+time;
        scene.glFunc.glPushMatrix();
        scene.setViewpoint(tt*0.2f, -tt*0.1f-1.0f, tt*0.5f+9.0f,  0, 0, -5.0f);
      }
    } else {
      scene.glFunc.glPushMatrix();
      scene.setViewpoint(0, -1.0f, 9.0f,  0, 0, -5.0f);
    }

    sIdx[0] = 0; sIdx[1] = OBJMAX; sIdx[2] = OBJMAX*2;
    setLayerdSprites(letters);
    setLayerdSprite(battery);
    setLayerdSprites(bullets);
    setLayerdSprites(enLines);
    setLayerdSprites(crafts);
    setLayerdSprites(shots);
    setLayerdSprites(lasers);
    setLayerdSprites(frames);
    setLayerdSprites(leftBtrs);

		ShapeProperties.setPolygonMode();
    for ( int i=2 ; i>=0 ; i-- ) {
      if ( i == 0 ) {
    		ShapeProperties.setLineMode();
      }
      for ( int j=sIdx[i]-1 ; j>=OBJMAX*i ; j-- ) {
        sprites[j].draw();
      }
    }
    scene.glFunc.glPopMatrix();
    waitFrame();
  }

  public final void addEngage(EngData ed) {
    sm.addEngage(ed);
  }

  private int bltIdx = BULLETMAX-1;

  public final Bullet addBullet(float x, float y, float layer, float deg,
   float speed, int shapeIdx, int elCnt) {
    for ( int i=BULLETMAX-1 ; i>=0 ; i--, bltIdx-- ) {
      if ( bltIdx < 0 ) bltIdx = BULLETMAX-1;
      if ( !bullets[bltIdx].exists() ) {
        bullets[bltIdx].start(x, y, layer, deg, speed, shapeIdx, elCnt);
        return bullets[bltIdx];
      }
    }
    if ( layer == 1 ) {
      for ( int i=BULLETMAX-1 ; i>=0 ; i--, bltIdx-- ) {
        if ( bltIdx < 0 ) bltIdx = BULLETMAX-1;
        if ( bullets[bltIdx].layer != 1 ) {
          bullets[bltIdx].start(x, y, layer, deg, speed, shapeIdx, elCnt);
          return bullets[bltIdx];
        }
      }
    }
    return null;
  }

  private int crtIdx = CRAFTMAX-1;

  public final void addCraft(EngData ed) {
    CraftGenerator cg = ed.cg;
    if ( ed.bssID == 1 ) {
      crafts[0].start(ed);
    } else {
      for ( int i=CRAFTMAX-2 ; i>=0 ; i--, crtIdx-- ) {
        if ( crtIdx < 1 ) crtIdx = CRAFTMAX-1;
        if ( !crafts[crtIdx].exists() ) {
          crafts[crtIdx].start(ed);
          break;
        }
      }
    }
  }

  private int shtIdx = SHOTMAX-1;

  public final void addShot(float x, float y) {
    int cn = 2;
    for ( int i=SHOTMAX-1 ; i>=0 ; i--, shtIdx-- ) {
      if ( shtIdx < 0 ) shtIdx = SHOTMAX-1;
      if ( !shots[shtIdx].exists() ) {
        shots[shtIdx].start(cn, x, y);
        cn--; if ( cn < 0 ) break;
      }
    }
  }

  public int lsrIdx = LASERMAX-1;

  public final void addLaser(float ofs) {
    for ( int i=LASERMAX-1 ; i>=0 ; i--, lsrIdx-- ) {
      if ( lsrIdx < 0 ) lsrIdx = LASERMAX-1;
      if ( !lasers[lsrIdx].exists() ) {
        lasers[lsrIdx].start(ofs);
        break;
      }
    }
  }

  private int elIdx = ELMAX-1;

  public final void addEnergyLine(Bullet blt1, Bullet blt2, int color) {
    for ( int i=ELMAX-1 ; i>=0 ; i--, elIdx-- ) {
      if ( elIdx < 0 ) elIdx = ELMAX-1;
      if ( !enLines[elIdx].exists() ) {
        enLines[elIdx].start(blt1, blt2, color);
        break;
      }
    }
  }

  private int ltIdx = LTMAX-1;

  public final Letters addLetters(String text, float x, float y, float layer,
   float size, int time) {
    for ( int i=LTMAX-1 ; i>=0 ; i--, ltIdx-- ) {
      if ( ltIdx < 0 ) ltIdx = LTMAX-1;
      if ( !letters[ltIdx].exists() ) {
        return letters[ltIdx].start(text, x, y, layer, size, time);
      }
    }
    return null;
  }

  public final void moveBossParts(int[] mvBssID, float mx, float my, float layer) {
    for ( int j=0 ; mvBssID[j]>0 ; j++ ) {
      int mbid = mvBssID[j];
      for ( int i=crafts.length-1 ; i>=0 ; i-- ) {
        if ( crafts[i].exists() && crafts[i].cPrp.bssID == mbid ) {
          crafts[i].x += mx; crafts[i].y += my;
          crafts[i].layer = layer;
        }
      }
    }
  }

  public final void destoryBossParts(int[] mvBssID) {
    for ( int j=0 ; mvBssID[j]>0 ; j++ ) {
      int mbid = mvBssID[j];
      for ( int i=crafts.length-1 ; i>=0 ; i-- ) {
        if ( crafts[i].exists() && crafts[i].cPrp.bssID == mbid ) {
          crafts[i].disappearMove();
        }
      }
    }
  }

  public final void madBossParts(int[] mdBssID) {
    for ( int j=0 ; mdBssID[j]>0 ; j++ ) {
      int mdid = mdBssID[j];
      for ( int i=crafts.length-1 ; i>=0 ; i-- ) {
        if ( crafts[i].exists() && crafts[i].cPrp.bssID == mdid ) {
          crafts[i].changeBltPtn();
        }
      }
    }
  }

  public final void startDatt(int dattID) {
    for ( int i=crafts.length-1 ; i>=0 ; i-- ) {
      if ( crafts[i].exists() && crafts[i].cPrp.dattID == dattID ) {
        crafts[i].startDattBns();
      }
    }
  }

  private void moveObjects(IMove[] objs) {
    for ( int i=objs.length-1 ; i>=0 ; i-- ) {
      if ( objs[i].exists() ) {
        objs[i].move();
      }
    }
  }

  public final void subLayer(LayerdSprite[] objs, float lyr) {
    for ( int i=objs.length-1 ; i>=0 ; i-- ) {
      objs[i].layer -= lyr;
    }
  }

  public final void subLayerCrafts(Craft[] crafts, float lyr) {
    if ( bssTime < 0 ) {
      for ( int i=crafts.length-1 ; i>=0 ; i-- ) {
        if ( crafts[i].exists() ) {
          crafts[i].layer -= lyr;
        }
      }
    } else {
      for ( int i=crafts.length-1 ; i>=0 ; i-- ) {
        if ( crafts[i].exists() && crafts[i].cPrp.bssID<=0 ) {
          crafts[i].layer -= lyr;
        }
      }
    }
  }

  public final void resetFrames(LayerdSprite[] objs) {
    for ( int i=objs.length-1 ; i>=0 ; i-- ) {
      objs[i].layer = i;
    }
  }

  public final void playSe(int nmb) {
    if ( !saveData.sef ) return;
    sePlayer.play(nmb);
  }

  private void moveLayer() {
    subLayer(bullets, battery.mvLyr);
    subLayer(enLines, battery.mvLyr);
    subLayerCrafts(crafts, battery.mvLyr);
    subLayer(frames, battery.mvLyr);
  }

  private void pauseWakeUp() {
    if ( sceneUI.psKey ) {
      sceneUI.psKey = false;
      pusLt.clear();
      mode = 0;
    }
  }

  private Letters cntCnt;
  private boolean arwPushed = false;
  private boolean btnPushed = false;

  private void gameoverWakeUp() {
    ttlTime++;
    if ( ttlTime < 0 ) {
      if ( sceneUI.upKey || sceneUI.dwKey || sceneUI.lfKey || sceneUI.rtKey ) {
        if ( !arwPushed ) {
          ttlTime -= (ttlTime%30);
          if ( ttlTime == 0 ) ttlTime = -1;
          arwPushed = true;
        }
      } else {
        arwPushed = false;
      }
      if ( sceneUI.stKey || sceneUI.lsKey ) {
        if ( !btnPushed ) {
          credit--;
          setLeftBattery(leftMax);
          battery.resetEnLine();
          score = 0; nextEtdScr = FIRSTEXTEND;
          stgstScore = 999999999;
          mode = 0;
          cntCnt.clear();
          stgLt.clear();
          btnPushed = true;
        }
      } else {
        btnPushed = false;
      }
      if ( (ttlTime%30)==0 ) {
        cntCnt.set(0, 0, 3);
        cntCnt.add(-(int)(ttlTime/30)-1);
      }
    } else if ( ttlTime == 0 ) {
      cntCnt.clear();
      stgLt.set();
      stgLt.add("GAME OVER");
    } else if ( ttlTime == 60 ) {
      stopBgm();
      startEntry();
    } else if ( ttlTime == 10030 ) {
      addLetters("YOU MADE IT.", 0, 0, 1, 0.6f, 99999);
    } else if ( ttlTime == 10060 ) {
      addLetters("LEFT BONUS", 0, -6, 1, 0.5f, 99999);
      addLetters(Integer.toString(left*200000), 0, -8, 1, 0.5f, 99999);
      score += left*200000;
      if ( score > gmMaxScr ) gmMaxScr = score;
      scrLt.set(0, 13, 0.6f);
      scrLt.add(score);
    } else if ( ttlTime == 10210 ) {
      startEntry();
    }
  }

  private void setNameLetter() {
    nmLt.set(-nmIdx*0.75f, 0, 1.5f);
    for ( int i=0 ; i<nmIdx ; i++ ) {
      nmLt.add(nmChrs[rnkNm[i]]);
    }
    if ( nmIdx < 3 ) {
      nmChrLt.set();
      nmChrLt.add(nmChrs[rnkNm[nmIdx]]);
    } else {
      nmChrLt.set();
      nmChrLt.add('_');
    }
  }

  private int rptCnt;

  private void entryWakeUp() {
    if ( ttlTime == 0 ) {
      if ( sceneUI.upKey || sceneUI.dwKey || sceneUI.lfKey ) {
        if ( !arwPushed ) {
          arwPushed = true;
          if ( sceneUI.upKey && nmIdx<3 ) {
            rnkNm[nmIdx]--; if ( rnkNm[nmIdx] >= 38 ) rnkNm[nmIdx] = 37;
            nmChrLt.set();
            nmChrLt.add(nmChrs[rnkNm[nmIdx]]);
          } else if ( sceneUI.dwKey && nmIdx<3 ) {
            rnkNm[nmIdx]++; if ( rnkNm[nmIdx] >= 38 ) rnkNm[nmIdx] = 0;
            nmChrLt.set();
            nmChrLt.add(nmChrs[rnkNm[nmIdx]]);
          } else if ( sceneUI.lfKey ) {
            if ( nmIdx > 0 ) nmIdx--;
            setNameLetter();
          }
        } else {
          rptCnt--;
          if ( rptCnt <= 0 ) {
            rptCnt = 3;
            arwPushed = false;
          }
        }
      } else {
        rptCnt = 32;
        arwPushed = false;
      }
      if ( sceneUI.stKey || sceneUI.lsKey ) {
        if ( !btnPushed ) {
          btnPushed = true;
          if ( nmIdx == 3 ) {
            ttlTime = 1;
            for ( int i=0 ; i<3 ; i++ ) {
              saveData.rnkNames[entRnk][i] = nmChrs[rnkNm[i]];
            }
            stgLt.clear();
            nmLt.clear(); nmChrLt.clear();
            rnkScrLt.clear();
            showRanking(60);
          } else {
            nmIdx++;
            if ( nmIdx < 3 ) rnkNm[nmIdx] = rnkNm[nmIdx-1];
            setNameLetter();
          }
        }
      } else {
        btnPushed = false;
      }
    } else {
      ttlTime++;
      if ( ttlTime > 60 ) startTitle();
    }
  }

  private void checkExtend() {
    if ( score >= nextEtdScr ) {
      left++; setLeftBattery(left);
      playSe(5);
      if ( nextEtdScr == FIRSTEXTEND ) nextEtdScr = SECONDEXTEND;
      else nextEtdScr = Integer.MAX_VALUE;
    }
  }

  private void stageWakeUp() {
    battery.move();
    if ( battery.mvLyr != 0 ) moveLayer();
    moveObjects(bullets);
    moveObjects(enLines);
    moveObjects(crafts);
    moveObjects(shots);
    moveObjects(lasers);
    moveObjects(letters);
    frames[FRMAX-2].move();
    frames[FRMAX-1].move();

    if ( pvScr != score ) {
      scrLt.set(0, 13, 0.6f);
      scrLt.add(score);
      checkExtend();
    }

    if ( time == 0 ) {
      sm.makeEngQue(battery.stgLyr, saveData.shwLyrNmb);
      resetFrames(frames);
    } else if ( time == -99900 ) {
      if ( mode == 1 ) {
        startTitle();
        return;
      }
      stage++;
      if ( stage > STAGENMB || stgSlcf ) {
        credit = -1;
        if ( !stgSlcf ) {
          startEnding();
        } else {
          startGameover();
        }
      } else {
        startStage();
        return;
      }
    }
    if ( time >= sm.nextEngTime ) {
      sm.addCrafts(time);
    }
    if ( bssTime >= stgEndTime ) {
      clearStage();
    }
    time++;
    bssTime++;
  }

  private void menuWakeUp() {
    frames[FRMAX-2].move();
    frames[FRMAX-1].move();
    if ( sceneUI.upKey || sceneUI.dwKey || sceneUI.lfKey || sceneUI.rtKey ) {
      if ( !arwPushed ) {
        int pmi = menuIdx;
        if ( sceneUI.upKey ) {
          menuIdx--;  if ( menuIdx < 0 ) menuIdx = MENUIDXMAX-1;
        } else if ( sceneUI.dwKey ) {
          menuIdx++; if ( menuIdx >= MENUIDXMAX ) menuIdx = 0;
        }
        switch ( menuIdx ) {
          case 1:
            if ( sceneUI.lfKey ) {
              selectedStage--;
              if ( selectedStage < 1 ) selectedStage = saveData.stgNmb;
              menuLetters[1].set();
              menuLetters[1].add("STAGE SELECT "+selectedStage);
            } else if ( sceneUI.rtKey ) {
              selectedStage++;
              if ( selectedStage > saveData.stgNmb ) selectedStage = 1;
              menuLetters[1].set();
              menuLetters[1].add("STAGE SELECT "+selectedStage);
            }
            break;
          case 2:
            if ( sceneUI.lfKey ) {
              saveData.shwLyrNmb--;
              if ( saveData.shwLyrNmb < 2 ) saveData.shwLyrNmb = 9;
              menuLetters[2].set();
              menuLetters[2].add("LAYER "+(saveData.shwLyrNmb-1));
            } else if ( sceneUI.rtKey ) {
              saveData.shwLyrNmb++;
              if ( saveData.shwLyrNmb > 9 ) saveData.shwLyrNmb = 2;
              menuLetters[2].set();
              menuLetters[2].add("LAYER "+(saveData.shwLyrNmb-1));
            }
            break;
          case 3:
            if ( sceneUI.lfKey ) {
              saveData.frameSkip--;
              if ( saveData.frameSkip < 0 ) saveData.frameSkip = 2;
              menuLetters[3].set();
              menuLetters[3].add("FRAME SKIP "+saveData.frameSkip);
            } else if ( sceneUI.rtKey ) {
              saveData.frameSkip++;
              if ( saveData.frameSkip > 2 ) saveData.frameSkip = 0;
              menuLetters[3].set();
              menuLetters[3].add("FRAME SKIP "+saveData.frameSkip);
            }
            break;
          case 4:
            if ( sceneUI.lfKey || sceneUI.rtKey ) {
              boolean joyf = !saveData.joystickf;
              if ( joyf ) {
                sceneUI.initJoystick();
                if ( sceneUI.joylibloaded ) {
                  saveData.joystickf = true;
                  menuLetters[4].set();
                  menuLetters[4].add("JOYSTICK ON");
                }
              } else {
                saveData.joystickf = false;
                menuLetters[4].set();
                menuLetters[4].add("JOYSTICK OFF");
              }
              sceneUI.enable(saveData.joystickf);
            }
            break;
          case 5:
            if ( sceneUI.lfKey || sceneUI.rtKey ) {
              saveData.bgmf = !saveData.bgmf;
              if ( saveData.bgmf ) {
                menuLetters[5].set();
                menuLetters[5].add("BGM ON");
              } else {
                menuLetters[5].set();
                menuLetters[5].add("BGM OFF");
              }
            }
            break;
          case 6:
            if ( sceneUI.lfKey || sceneUI.rtKey ) {
              saveData.sef = !saveData.sef;
              if ( saveData.sef ) {
                menuLetters[6].set();
                menuLetters[6].add("SE ON");
              } else {
                menuLetters[6].set();
                menuLetters[6].add("SE OFF");
              }
            }
            break;
        }
        if ( pmi != menuIdx ) {
          menuLetters[menuIdx].setSize(0.8f);
          menuLetters[pmi].setSize(0.5f);
        }
        arwPushed = true;
      }
    } else {
      arwPushed = false;
    }
    if ( sceneUI.stKey || sceneUI.lsKey ) {
      if ( !btnPushed ) {
        switch ( menuIdx ) {
          case 0:
            startGame();
            break;
          case 1:
            startGame(selectedStage);
            break;
          case 7:
            destroy();
            break;
        }
      }
    } else {
      btnPushed = false;
    }
  }

  final private String[] rnkStrs = {
    "1ST", "2ND", "3RD", "4TH", "5TH", "6TH", "7TH", "8TH", "9TH", "10TH"};

  private void showRanking(int tm) {
    for ( int i=0 ; i<10 ; i++ ) {
      float ly = -i*2+9;
      float lx = -6.5f; if ( i == 9 ) lx = -7.1f;
      addLetters(rnkStrs[i]+" "+new String(saveData.rnkNames[i]), lx, ly, 1, 0.7f, tm);
      addLetters(Integer.toString(saveData.rnkScores[i]), 6, ly, 1, 0.7f, tm);
    }
  }

  private void titleWakeUp() {
    stageWakeUp();
    ttlTime++;
    if ( ttlTime == 90 ) {
      stgLt.x = -14; stgLt.y = 13.6f;
      stgLt.xScl = 1f; stgLt.yScl = 1f;
    } else if ( ttlTime == 180 ) {
      showRanking(420);
    }
    if ( sceneUI.stKey || sceneUI.lsKey ) startMenu();
  }

  private void gameWakeUp() {
    stageWakeUp();
    if ( battery.stgLyr==0 && time<120 ) {
      if ( time > 10 && time < 30 ) {
        stgLt.x -= 0.1f; stgLt.y -= 0.5f;
        stgLt.xScl -= 0.04f; stgLt.yScl -= 0.04f;
      } else if ( time == 30 ) {
        stgLt2 = addLetters("", -6f, -11f, 1, 0.3f, 90);
        stgMsgIdx = 0;
      } else if ( time>30 && (time%3)==0 && stgMsgIdx<stageMessage.length() ) {
        stgLt2.add(stageMessage.charAt(stgMsgIdx));
        stgMsgIdx++;
      }
    }
    if ( sceneUI.psKey ) {
      sceneUI.psKey = false;
      startPause();
    }
  }

  public final void wakeUp() {
    frmCnt++;
    if ( frmCnt > saveData.frameSkip ) {
      scene.display();
      frmCnt = 0;
    } else {
      waitFrame();
    }
    if ( shakef ) LayerdSprite.shake(bssTime);
    sceneUI.checkJoystick();

    switch ( mode ) {
      case 0:
        gameWakeUp();
        break;
      case 1:
        titleWakeUp();
        break;
      case 2:
        gameoverWakeUp();
        break;
      case 3:
        menuWakeUp();
        break;
      case 4:
        pauseWakeUp();
        break;
      case 5:
        entryWakeUp();
        break;
    }
    if ( sceneUI.ttKey ) {
      sceneUI.ttKey = false;
      stopBgm();
      startTitle();
    }
  }

  static private boolean keyPriorityf = false;

  public static void main(String[] args) {
    if ( GLContext.loadNativeLibraries(null, null, null) == false ) {
      System.out.println("Comraid: can't load native libraries");
      return;
    }
		GLContext.gljNativeDebug = false;
		GLContext.gljClassDebug  = false;
    for ( int i=0 ; i<args.length ;i++ ) {
      if ( args[i].equals("-keyPriority") ) {
        keyPriorityf = true;
      }
    }

    Comraid comraid = new Comraid();
    comraid.start();
  }
}
