package jp.gr.java_conf.abagames.comraid;

import jp.gr.java_conf.abagames.util.gl4j.*;
import jp.gr.java_conf.abagames.util.*;
import gl4java.*;
import java.util.Random;

/**
  Enemy Craft operation
  */
public class Craft extends LayerdSprite implements IMove {
  static private Random rnd = new Random();

  final private int MDMAX = 32, BDMAX = 360, BDMAXBSS = 2048, CDMAX = 10;
  final private int SHOTDMG = 2, LASERDMG = 3;

  private boolean existf;
  // Maneuver data
  private MnvData[] mnvData = new MnvData[MDMAX];
  private int mnvIdx, nextMnvTime;
  // Bullet firing data
  private BltData[] bltData;
  // Collision data
  private ClsData[] clsData = new ClsData[CDMAX];
  private int bltIdx, nextBltTime;
  public EngData cPrp;
  private float mx, my, mlayer, mdeg;
  private boolean degAim;
  private int shield;
  private int dattID;
  private int time, bltTime;
  // damaged flush counter
  private int flsCnt;
  // disappeard counter
  private int dapCnt;
  // flags for next engamement
  private boolean linedEng, nextEng;
  // score 4 [dst at a time] [quick sht]
  private int bnScr;
  private Letters lt;
  private int dmShTime;

  private Comraid comraid;
  private Battery aim;

  public Craft(GLFunc gl, Comraid comraid, Battery aim, boolean bsf) {
    super(gl);
    for ( int i=0 ; i<MDMAX ; i++ ) mnvData[i] = new MnvData();
    for ( int i=0 ; i<CDMAX ; i++ ) clsData[i] = new ClsData();
    if ( bsf ) {
      bltData = new BltData[BDMAXBSS];
      for ( int i=0 ; i<BDMAXBSS ; i++ ) bltData[i] = new BltData();
    } else {
      bltData = new BltData[BDMAX];
      for ( int i=0 ; i<BDMAX ; i++ ) bltData[i] = new BltData();
    }
    this.comraid = comraid;
    this.aim = aim;
    existf = false;
  }

  final public void start(EngData ed) {
    if ( ed.delaydApp && comraid.bssTime < 0 ) {
      comraid.addEngage(ed);
      return;
    }
    cPrp = ed;
    CraftGenerator cg = ed.cg;
    shape = cg.shape;
    cg.setEngData(ed);
    cg.makeManeuver(mnvData);
    cg.makeBulletData(bltData);
    cg.makeCollisionData(clsData);
    shield = cPrp.shield;
    dattID = cPrp.dattID;
    x = mnvData[0].x; y = mnvData[0].y;
    layer = mnvData[0].layer; deg = mnvData[0].deg;
    time = bltTime = 0;
    flsCnt = -1;
    dapCnt = -1;
    xScl = yScl = 1.0f; dstDeg = 0;
    bnScr = -99999;
    mnvIdx = 0;
    setNextManueuver();
    bltIdx = 0; nextBltTime = bltData[0].time;
    red = 0.2f; green = 1.0f; blue = 0.2f;
    linedEng = nextEng = false;
    existf = true;
  }

  public void clear() {
    existf = false;
  }

  private void changeNextMnv() {
    while ( mnvData[mnvIdx].time != -1 ) mnvIdx++;
    mnvIdx++;
    time = 0;
    setNextManueuver();
  }

  private void setNextManueuver() {
    x = mnvData[mnvIdx].x; y = mnvData[mnvIdx].y;
    layer = mnvData[mnvIdx].layer;
    if ( mnvData[mnvIdx].deg >= 0 ) {
      deg = mnvData[mnvIdx].deg;
    }
    mnvIdx++;

    if ( mnvData[mnvIdx].layer == -99999 ) {
      time = (int)mnvData[mnvIdx].x;
      for ( ; mnvIdx>=0 ; mnvIdx-- ) {
        if ( mnvData[mnvIdx].time < time ) {
          break;
        }
      }
      mnvIdx++;
      nextMnvTime = mnvData[mnvIdx].time;
      return;
    }

    nextMnvTime = mnvData[mnvIdx].time;
    int ts = nextMnvTime - time;
    mx = (mnvData[mnvIdx].x-x)/ts; my = (mnvData[mnvIdx].y-y)/ts;
    mlayer = (mnvData[mnvIdx].layer-layer)/ts;
    if ( mnvData[mnvIdx].deg < 0 ) {
      mdeg = -mnvData[mnvIdx].deg;
      degAim = true;
    } else {
      mdeg = (mnvData[mnvIdx].deg-deg);
      if ( mdeg > 180 ) mdeg -= 360;
      else if ( mdeg < -180 ) mdeg += 360;
      mdeg /= ts;
      degAim = false;
    }
    nextMnvTime = mnvData[mnvIdx].time;
  }

  final public void changeBltPtn() {
    for ( ; bltData[bltIdx].type != -1 ; bltIdx++ ) {}
    bltIdx++;
    bltTime = 0;
    nextBltTime = bltData[bltIdx].time;
  }

  private void fireBullet() {
    boolean nelf = false, self;
    int elCnt = 0;
    Bullet blt1, blt2 = null;
    while ( nextBltTime == bltTime ) {
      float ofsX = bltData[bltIdx].ofsX, ofsY = bltData[bltIdx].ofsY;
      float ofx = x, ofy = y, sd, cd;
      if ( ofsX!=0 || ofsY!=0 ) {
        sd = SCTable.sin[(int)deg];
        cd = SCTable.cos[(int)deg];
        ofx += ofsX*cd-ofsY*sd;
        ofy += ofsX*sd+ofsY*cd;
      }
      float fDeg = 0;
      switch ( bltData[bltIdx].type ) {
        case 0:
          float ad = DegUtil.getDeg(aim.x-ofx, aim.y-ofy);
          fDeg = ad+bltData[bltIdx].ofsDeg;
          if ( fDeg < 0 ) fDeg += 360;
          else if ( fDeg >= 360 ) fDeg -= 360;
          break;
        case 1:
          fDeg = deg + bltData[bltIdx].ofsDeg;
          if ( fDeg < 0 ) fDeg += 360;
          else if ( fDeg >= 360 ) fDeg -= 360;
          break;
        case 2:
          fDeg = bltData[bltIdx].ofsDeg;
          break;
        case -1:
          bltTime = (int)bltData[bltIdx].ofsDeg;
          bltIdx--;
          for ( ; bltIdx>=0 ; bltIdx-- ) {
            if ( bltData[bltIdx].time < bltTime || bltData[bltIdx].type == -1 ) {
              break;
            }
          }
          bltIdx++;
          nextBltTime = bltData[bltIdx].time;
          return;
      }
      self = nelf;
      nelf = false;
      if ( bltData[bltIdx].engLine!=-1 && layer==1 ) {
        nelf =  true;
      }
      if ( self ) {
        if ( nelf ) elCnt = 2;
        else elCnt = 1;
      } else if ( nelf ) {
        elCnt = 1;
      }
      blt1 = comraid.addBullet(ofx, ofy, layer, fDeg,
       bltData[bltIdx].speed, bltData[bltIdx].shapeIdx, elCnt);
      if ( self ) {
        comraid.addEnergyLine(blt1, blt2, bltData[bltIdx-1].engLine);
      }
      if ( nelf ) {
        blt2 = blt1;
      }
      bltIdx++;
      nextBltTime = bltData[bltIdx].time;
    }
  }

  private int md;

  public void disappearMove() {
    dapCnt = 30; md = (rnd.nextInt()&15)+15;
    layer = 1.01f;
    shield = 99999;
  }

  private void disappear() {
    comraid.score += cPrp.score*cPrp.scrTimes;
    if ( bnScr > 0 ) {
      comraid.score += bnScr*cPrp.scrTimes;
      if ( cPrp.scrTimes > 1 ) {
        lt.set(x, y, 0.8f);
        lt.add(bnScr);
        lt.add('X');
        lt.add(cPrp.scrTimes);
      } else {
        lt.set(x, y, 0.8f);
        lt.add(bnScr);
      }
      lt.time = 60;
      comraid.playSe(1);
    } else if ( cPrp.scrTimes > 1 ) {
      lt = comraid.addLetters("X"+Integer.toString(cPrp.scrTimes),
       x, y, layer, 0.8f, 60);
      comraid.playSe(2);
    } else {
      comraid.playSe(0);
    }
    if ( dattID > 0 ) {
      comraid.startDatt(dattID);
    }
    if ( cPrp.bssID > 0 ) {
      comraid.destoryBossParts(cPrp.mvBssID);
      comraid.madBossParts(cPrp.mdBssID);
      if ( cPrp.bssID == 1 ) {
        comraid.clearStage();
      }
    }
    cPrp.scrTimes *= 2;
    disappearMove();
  }

  final private int HITSCR = 10;

  private void damage() {
    comraid.score += HITSCR;
    red = 1.0f; blue = 1.0f; flsCnt = 2;
    if ( cPrp.bssID > 0 ) {
      if ( dmShTime <= 0 ) {
        lt = comraid.addLetters(Integer.toString(shield),
         x, y, 1, 0.5f, 30);
      } else {
        lt.set(x, y, 0.5f);
        lt.add(shield);
        lt.time = 30;
      }
      dmShTime = 30;
    }
  }

  private void hitShot(Shot st) {
    shield -= SHOTDMG;
    if ( shield <= 0 ) {
      disappear();
    } else {
      damage();
    }
    st.hit();
  }

  private void hitLaser(Laser ls, Laser[] lasers, int lIdx) {
    shield -= LASERDMG;
    if ( shield <= 0 ) {
      disappear();
    } else {
      damage();
    }
    float hy = ls.y;
    for ( int i=lasers.length-1 ; i>=0 ; i--, lIdx++ ) {
      if ( lIdx >= comraid.LASERMAX ) lIdx = 0;
      //if ( lasers[i].y >= hy ) lasers[i].hit();
      if ( !lasers[lIdx].isValid ) break;
      lasers[lIdx].hit();
    }
  }

  final private float BTCLS = -1.0f;

  private void checkCollision() {
    float ofx, ofy, sd, cd;
    for ( int j=0 ; clsData[j].width > 0 ; j++ ) {
      if ( j != 0 ) {
        float cx = clsData[j].x, cy = clsData[j].y;
        sd = SCTable.sin[(int)deg];
        cd = SCTable.cos[(int)deg];
        ofx = x + cx*cd-cy*sd;
        ofy = y + cx*sd+cy*cd;
      } else {
        ofx = x; ofy = y;
      }
      float cw = clsData[j].width, ch = clsData[j].height;
      for ( int i=comraid.SHOTMAX-1 ; i>=0 ; i-- ) {
        Shot st = comraid.shots[i];
        if ( st.isValid ) {
          if ( Math.abs(st.x-ofx)<=cw
           && Math.abs(st.y-ofy)<=ch ) {
            hitShot(st);
            break;
          }
        }
      }
      int lIdx = comraid.lsrIdx;
      for ( int i=comraid.LASERMAX-1 ; i>=0 ; i--, lIdx++ ) {
        if ( lIdx >= comraid.LASERMAX ) lIdx = 0;
        Laser ls = comraid.lasers[lIdx];
        if ( ls.isValid ) {
          if ( Math.abs(ls.x-ofx)<=cw-(1-ls.xScl)
           && Math.abs(ls.y-ofy)<=ch ) {
            hitLaser(ls, comraid.lasers, lIdx);
            break;
          }
        }
      }
      if ( Math.abs(aim.x-ofx)<=cw+BTCLS
       && Math.abs(aim.y-ofy)<=ch+BTCLS ) {
        aim.hit();
      }
    }
  }

  private void startBns() {
    bnScr = cPrp.stScr + cPrp.decScr;
    if ( bnScr <= 0 ) return;
    lt = comraid.addLetters(Integer.toString(bnScr), x, y, layer, 0.5f, 1);
    if ( lt == null ) bnScr = 0;
  }

  final public void startDattBns() {
    dattID = -1;
    startBns();
  }

  final public void move() {
    x += mx; y += my; layer += mlayer;
    if ( degAim ) {
      float amd = DegUtil.getDeg(aim.x-x, aim.y-y);
      float ad = deg-amd;
      if ( ad > 180 ) ad -= 360;
      else if ( ad<-180 ) ad += 360;
      if ( ad > mdeg ) deg -= mdeg;
      else if ( ad > -mdeg ) deg = amd;
      else deg += mdeg;
    } else {
      deg += mdeg;
    }
    if ( cPrp.bssID > 0 ) {
      comraid.moveBossParts(cPrp.mvBssID, mx, my, layer);
      if ( !cPrp.delaydApp && comraid.bssTime>=0 && layer == 1 ) {
        cPrp.delaydApp = true;
        changeNextMnv();
      }
      dmShTime--;
    }
    if ( deg < 0 ) deg += 360;
    else if ( deg >= 360 ) deg -= 360;
    if ( dapCnt < 0 ) {
      if ( !linedEng ) {
        if ( layer == 2 ) {
          cPrp.shield = shield;
          comraid.addEngage(this.cPrp);
          linedEng = true;
          nextEng = false;
        } else if ( !nextEng ) {
          if ( layer >= 1 ) {
            nextEng = true;
          }
        }
      }
      time++; bltTime++;
      if ( time >= nextMnvTime ) {
        setNextManueuver();
        if ( nextMnvTime < 0 ) {
          if ( nextEng ) {
            comraid.addEngage(this.cPrp);
          }
          existf = false;
        }
      }
      if ( bltTime >= nextBltTime ) {
        fireBullet();
      }
      if ( layer == 1 ) {
        if ( bnScr == -99999 ) {
          if ( dattID <= 0 ) {
            startBns();
          } else {
            bnScr = 0;
          }
        }
        if ( bnScr > 0 ) {
          bnScr -= cPrp.decScr;
          if ( bnScr > 0 ) {
            lt.set(x, y, 0.5f);
            lt.add(bnScr);
            lt.time = 1;
          }
        }
        checkCollision();
      }
    } else {
      dstDeg += md;
      if ( dapCnt > 24 ) {
        xScl *= 1.4f; yScl *= 0.8f;
      } else {
        xScl = 0.9f; yScl *= 1.1f;
      }
      dapCnt--;
      if ( dapCnt <= 0 ) existf = false;
      blue = dapCnt*0.025f; red = green = dapCnt*0.032f;
    }
    if ( flsCnt > 0 ) {
      flsCnt--;
      if ( flsCnt == 0 ) {
        red = 0.2f; green = 1.0f; blue = 0.2f;
      }
    }
  }

  final public boolean exists() {
    return existf;
  }
}

