package jp.gr.java_conf.abagames.comraid;

import jp.gr.java_conf.abagames.comraid.shapes.*;
import jp.gr.java_conf.abagames.util.gl4j.*;
import jp.gr.java_conf.abagames.util.*;
import gl4java.*;

/**
  Spark operation
  */
public class EnergyLine extends LayerdSprite implements IMove {
  static final public int RED = 0, BLUE = 1, YELLOW = 2;

  private boolean existf;
  private Bullet blt1, blt2;
  private int color;
  private Battery btr;

  public EnergyLine(GLFunc gl, Battery btr) {
    super(gl);
    this.btr = btr;
    existf = false;
    shape = new EnergyLineShape();
  }

  private float[] or = new float[2];
  private float[] og = new float[2];
  private float[] ob = new float[2];

  final public void start(Bullet blt1, Bullet blt2, int color) {
    if ( blt1 == null || blt2 == null ) return;
    this.blt1 = blt1; this.blt2 = blt2;
    this.color = color;
    switch ( color ) {
      case RED:
        or[0]= 1; og[0] = 0.3f; ob[0] = 0.3f;
        or[1]= 1; og[1] = 0.6f; ob[1] = 0.6f;
        break;
      case BLUE:
        or[0]= 0.6f; og[0] = 0.6f; ob[0] = 1;
        or[1]= 0.8f; og[1] = 0.8f; ob[1] = 1;
        break;
      case YELLOW:
        or[0]= 1; og[0] = 1; ob[0] = 0.3f;
        or[1]= 1; og[1] = 1; ob[1] = 0.6f;
        break;
    }
    layer = blt1.layer;
    ((EnergyLineShape)shape).setPosition(blt1.x, blt1.y, blt2.x, blt2.y);
    existf = true;
  }

  public void clear() {
    existf = false;
  }

  private int clCnt;
  private float CLSWDT = 1;
  private float CLSWDTPOW = (CLSWDT/2)*(CLSWDT/2);

  final public void move() {
    ((EnergyLineShape)shape).setPosition(blt1.x, blt1.y, blt2.x, blt2.y);
    if ( (blt1.x>Comraid.frmWidth || blt1.x<-Comraid.frmWidth
     || blt1.y>Comraid.frmHeight || blt1.y<-Comraid.frmHeight)
     && (blt2.x>Comraid.frmWidth || blt2.x<-Comraid.frmWidth
     || blt2.y>Comraid.frmHeight || blt2.y<-Comraid.frmHeight) ) {
      blt1.disappearEngLine();
      blt2.disappearEngLine();
      existf = false;
      return;
    }
    int i = ((clCnt++)&1);
    red = or[i]; green = og[i]; blue = ob[i];
    if ( btr.dapCnt>=0 || layer!=1 ) return;
    float lx, ly, gx, gy;
    if ( blt1.x < blt2.x ) {
      lx = blt1.x; gx = blt2.x;
    } else {
      gx = blt1.x; lx = blt2.x;
    }
    float wdt = gx-lx;
    if ( wdt < CLSWDT ) {
      wdt = (CLSWDT-wdt)/2;
      gx += wdt; lx -= wdt;
    }
    if ( blt1.y < blt2.y ) {
      ly = blt1.y; gy = blt2.y;
    } else {
      gy = blt1.y; ly = blt2.y;
    }
    float hgt = gy - ly;
    if ( hgt < CLSWDT ) {
      hgt = (CLSWDT-hgt)/2;
      gy += hgt; ly -= hgt;
    }
    if ( (btr.x>lx && btr.x<gx) && (btr.y>ly && btr.y<gy) ) {
      float bx = btr.x - blt1.x, by = btr.y - blt1.y;
      float ax = blt2.x - blt1.x, ay = blt2.y - blt1.y;
      float ab = ax*bx+ay*by;
      float dst = (bx*bx+by*by) - (ab*ab/(ax*ax+ay*ay));
      if ( dst < CLSWDTPOW ) {
        btr.passEngLine(color);
        blt1.disappearEngLine();
        blt2.disappearEngLine();
        existf = false;
      }
    }
  }

  final public boolean exists() {
    return existf;
  }
}

