package jp.gr.java_conf.abagames.comraid;

import jp.gr.java_conf.abagames.util.gl4j.*;
import gl4java.*;
import Joystick;
import java.awt.*;
import java.awt.event.*;

/**
  Scene User Interface
  */
public class SceneUI extends Frame implements Runnable {
  private Comraid comraid;
  private Scene scene;

  public boolean upKey, dwKey, lfKey, rtKey, stKey, lsKey;
  public boolean psKey, ttKey;
  private boolean uKey, dKey, lKey, rKey;
  private boolean sKey, l2Key;
  private boolean nKey1, nKey2, nKey3, nKey4, nKey5, nKey6, nKey7, nKey8 ,nKey9;

  private EventQueue eventQueue;

  public SceneUI(Comraid comraid, Rectangle rct) {
    this.comraid = comraid;
    Toolkit tk = Toolkit.getDefaultToolkit();
    eventQueue = tk.getSystemEventQueue();
    try  {
      jbInit(rct);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit(Rectangle rct) throws Exception {
    setBounds(rct);
    setTitle("Comraid");
    scene = new Scene(rct.width, rct.height, comraid);
    scene.addKeyListener(new java.awt.event.KeyAdapter() {
      public final void keyPressed(KeyEvent e) {
        scene_keyPressed(e);
      }
      public final void keyReleased(KeyEvent e) {
        scene_keyReleased(e);
      }
    });
    add(scene, BorderLayout.CENTER);

    this.addWindowListener(new java.awt.event.WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        this_windowClosing(e);
      }
    });
  }

  public void run() {
  }

  public void procAllEvent() {
    try {
      eventQueue.invokeAndWait(this);
    } catch ( Exception e ) {
      e.printStackTrace();
    }
  }

  public Scene getScene() {
    return scene;
  }

  void this_windowClosing(WindowEvent e) {
    comraid.destroy();
  }

  private void scene_keyPressed(KeyEvent e) {
    switch ( e.getKeyCode() ) {
      case KeyEvent.VK_UP:
        uKey = true; return;
      case KeyEvent.VK_DOWN:
        dKey = true; break;
      case KeyEvent.VK_RIGHT:
        rKey = true; break;
      case KeyEvent.VK_LEFT:
        lKey = true; break;
      case KeyEvent.VK_NUMPAD1:
        nKey1 = true; break;
      case KeyEvent.VK_NUMPAD2:
        nKey2 = true; break;
      case KeyEvent.VK_NUMPAD3:
        nKey3 = true; break;
      case KeyEvent.VK_NUMPAD4:
        nKey4 = true; break;
      case KeyEvent.VK_NUMPAD6:
        nKey6 = true; break;
      case KeyEvent.VK_NUMPAD7:
        nKey7 = true; break;
      case KeyEvent.VK_NUMPAD8:
        nKey8 = true; break;
      case KeyEvent.VK_NUMPAD9:
        nKey9 = true; break;
      case KeyEvent.VK_X:
      case KeyEvent.VK_ALT:
        sKey = true; break;
      case KeyEvent.VK_Z:
      case KeyEvent.VK_CONTROL:
        l2Key = true; break;
      case KeyEvent.VK_ESCAPE:
      case KeyEvent.VK_F10:
        comraid.destroy(); break;
      case KeyEvent.VK_F9:
        ttKey = true; break;
      case KeyEvent.VK_F3:
        psKey = true; break;
    }
  }

  private void scene_keyReleased(KeyEvent e) {
    switch ( e.getKeyCode() ) {
      case KeyEvent.VK_UP:
        uKey = false; break;
      case KeyEvent.VK_DOWN:
        dKey = false; break;
      case KeyEvent.VK_RIGHT:
        rKey = false; break;
      case KeyEvent.VK_LEFT:
        lKey = false; break;
      case KeyEvent.VK_NUMPAD1:
        nKey1 = false; break;
      case KeyEvent.VK_NUMPAD2:
        nKey2 = false; break;
      case KeyEvent.VK_NUMPAD3:
        nKey3 = false; break;
      case KeyEvent.VK_NUMPAD4:
        nKey4 = false; break;
      case KeyEvent.VK_NUMPAD6:
        nKey6 = false; break;
      case KeyEvent.VK_NUMPAD7:
        nKey7 = false; break;
      case KeyEvent.VK_NUMPAD8:
        nKey8 = false; break;
      case KeyEvent.VK_NUMPAD9:
        nKey9 = false; break;
      case KeyEvent.VK_X:
      case KeyEvent.VK_ALT:
        sKey = false; break;
      case KeyEvent.VK_Z:
      case KeyEvent.VK_CONTROL:
        l2Key = false; break;
    }
  }

  public boolean joyf = false, joylibloaded = false;
  private Joystick js;

  public void initJoystick() {
    if ( joylibloaded ) return;
    try {
  		System.loadLibrary("joystick");
      js = new Joystick(JOYID);
      joylibloaded = true;
    } catch ( Exception e ) {
      System.out.println("Joystick is disable");
    }
  }

  public void enable(boolean joyf) {
    if ( !joylibloaded ) {
      this.joyf = false;
    } else {
      this.joyf = joyf;
    }
  }

  final private float JOYASB = 0.2f;
  final private int JOYID = 0;
  private boolean rtJoy, lfJoy, upJoy, dwJoy, stJoy, lsJoy;

  public final void checkJoystick() {
    if ( joyf ) {
      float x = js.getXPos();
      float y = js.getYPos();
      rtJoy = lfJoy = false;
      if ( x > JOYASB ) rtJoy = true;
      else if ( x < -JOYASB ) lfJoy = true;
      upJoy = dwJoy = false;
      if ( y > JOYASB ) dwJoy = true;
      else if ( y < -JOYASB ) upJoy = true;
      int btn = js.getButtons();
      stJoy = false;
      if ( (btn & js.BUTTON2)!=0 ) {
        stJoy = true;
      }
      lsJoy = false;
      if ( (btn & js.BUTTON1)!=0 ) {
        lsJoy = true;
      }
    }
    upKey = uKey | nKey7 | nKey8 | nKey9 | upJoy;
    dwKey = dKey | nKey1 | nKey2 | nKey3 | dwJoy;
    rtKey = rKey | nKey3 | nKey6 | nKey9 | rtJoy;
    lfKey = lKey | nKey1 | nKey4 | nKey7 | lfJoy;
    stKey = sKey | stJoy;
    lsKey = l2Key | lsJoy;
  }
}

